Package: BiocSingular
Version: 1.28.0
Date: 2025-10-23
Title: Singular Value Decomposition for Bioconductor Packages
Authors@R: c(person("Aaron", "Lun", role=c("aut", "cre", "cph"),
        email="infinite.monkeys.with.keyboards@gmail.com"))
Imports: BiocGenerics, S4Vectors, Matrix, methods, utils, DelayedArray,
        BiocParallel, ScaledMatrix, irlba, rsvd, Rcpp, beachmat (>=
        2.25.1)
Suggests: testthat, BiocStyle, knitr, rmarkdown, ResidualMatrix
biocViews: Software, DimensionReduction, PrincipalComponent
Description: Implements exact and approximate methods for singular value
        decomposition and principal components analysis, in a framework that
        allows them to be easily switched within Bioconductor packages or 
        workflows. Where possible, parallelization is achieved using 
        the BiocParallel framework.
License: GPL-3
LinkingTo: Rcpp, beachmat, assorthead
VignetteBuilder: knitr
SystemRequirements: C++17
RoxygenNote: 7.3.3
Encoding: UTF-8
BugReports: https://github.com/LTLA/BiocSingular/issues
URL: https://github.com/LTLA/BiocSingular
git_url: https://git.bioconductor.org/packages/BiocSingular
git_branch: RELEASE_3_23
git_last_commit: 675ff54
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 20:54:02 UTC; biocbuild
Author: Aaron Lun [aut, cre, cph]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
