# Singular value decomposition for Bioconductor

|Build|Status|
|-----|----|
| Bioc-release | [![](https://bioconductor.org/shields/build/release/bioc/BiocSingular.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/BiocSingular) |
| Bioc-devel   | [![](https://bioconductor.org/shields/build/devel/bioc/BiocSingular.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/BiocSingular) | 

This package provides a consistent class interface for singular value decompositions and principal components analysis for use throughout the Bioconductor ecosystem.
Check out the user guide on the [Bioconductor landing page](https://bioconductor.org/packages/devel/bioc/html/BiocSingular.html) for more details.
